 /*

********************************************************************************
**      ATI Mach 64 chipset driver
********************************************************************************
**
    Copyright (C)       1998    Isaac Connor
                                Mathew Wilkins

********************************************************************************
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
********************************************************************************

*/
#include "ATI/mach64.h"
#include "ATI/mach64_io.h"

#include "ATI/mach64_init.h"
/*--------
 *	Nab the startup register state.
 */

int mach64_init( struct kgi_display *dpy ) {
	int temp;
	GET_PRIVATE;


	/* test the linear framebuffer I/O mapping 
	   if there's a problem with the big aperture MMIO
	   mapping, it will probably show up here first. */

	mach64_out32(Scratch_Reg1, 0xAAAAAAAA, dpy);
	printk("wrote 0xAAAAAAAA, got back 0x%8x.\n",
			mach64_in32(Scratch_Reg1, dpy));
	
	mach64_out32(Scratch_Reg0, 0x55555555, dpy);
	printk("wrote 0x55555555, got back 0x%8x.\n",
			mach64_in32(Scratch_Reg0, dpy));


	/* important :) */
	dpy->mode.mmio  = &graphic_mmio_linear_fb;
	dpy->mode.fb = (void*) (card->LFB_VBase);
	return	EOK;
}

/*--------
 *	restore card to startup state
 */

int	mach64_restore_card( struct kgi_display *dpy ) {

	GET_PRIVATE;
	printk("Restoring card to original state\n");

	mach64_out32( Crtc_H_Total, card->crtc_h_total ,dpy);
	mach64_out32( Crtc_H_Sync, card->crtc_h_sync ,dpy);
	mach64_out32( Crtc_V_Total, card->crtc_v_total ,dpy);
	mach64_out32( Crtc_V_Sync, card->crtc_v_sync ,dpy);
	mach64_out32( Crtc_Vline, card->crtc_vline ,dpy);
	mach64_out32( Crtc_Display_Start, card->crtc_display_start ,dpy);
	mach64_out32( Crtc_Int_Cntl, card->crtc_int_cntl ,dpy);
	mach64_out32( Crtc_Gen_Cntl, card->crtc_gen_cntl ,dpy);
	mach64_out32( Clock_Cntl, card->clock_cntl ,dpy);
	mach64_out32( Bus_Cntl, card->bus_cntl ,dpy);
	mach64_out32( Mem_Cntl, card->mem_cntl ,dpy);
	mach64_out32( Dac_Cntl, card->dac_cntl ,dpy);
	mach64_out32( Gen_Test_Cntl, card->gen_test_cntl ,dpy);
	
	mem_free_region(&Mach64_LFB_Mem);
	kgi_free_small(card);
	return	1;
}




/*--------
 *	find out what grfx mode we are in.
 *	the different get state routines fill in Mach64_Default_Ggi_Mode
 *	and Mach64_Default_Kgi_Mode structures.
 *	and fill in dpy->mode ( the current kgi_mode structure )
 */

int	Mach64_Construct_Default_State( struct kgi_display *dpy ) {
/*    Mach64_Get_VGA_State( dpy );
*/
    return EOK;
}

