#define DEBUG_LEVEL     0

#define DRIVER_NAME "3DFX Banshee Integrated Clock driver"
#define DRIVER_REV  "0.1"

#include <kgi/module.h>
#include <3dfx/banshee.h>

#include "pll.h"

static const struct pll_limits dclk_pll =
{
	14318180,                       /* Fref                         */
	{ 1, 1 },                       /* Factor                       */
	{ 1000000, BANSHEE_MAX_CLOCK },       /* FVCO range                   */
	{ 0,3 },                       /* Post-VCO divider range       */
	{ 3, 257 },                     /* Dividend range               */
	{ 3, 65 }                       /* Divisor range                */
};
 
static struct kgi_clock clock =
{
  "3DFX",			/* Manufacturer         */
  "Banshee",			/* Model                */
  GM_8BIT | GM_15BIT | GM_16BIT | GM_24BIT | GM_32BIT,                       /* Supported modes      */
  0,                            /* Special capabilities */
  { 2048, 2048 },               /* Maximum resolution   */
  NULL,				/* Private data hook	*/
  CLK_TYPE_PROG,                /* Clock type           */
  {{
    0, 270000000,		/* Clock range          */
  }}
};

/* An SDAC command word is as follows:
 * **      [0:1]     s->p
 * **      [2:7]     s->div -2
 * **      [8:15]    s->mul -2
 * **      [16]      = test (0=normal, 1=CLK is output of VCO)
 * */

#define INVALID_COMMAND ((uint) -1)

static kgi_uint
pll_command(struct pll_state *s, struct kgi_mode *mode)
{
  kgi_uint res ; 
  DEBUG1("IN pll_command") ;
  ASSERT(s->mul - 2 < 256);
  ASSERT(s->div - 2 < 64);
  ASSERT(s->p < 4);
  DEBUG1("$$$$$$$$$$$ PLL mul=%d div=%d p=%d",s->mul,s->div,s->p) ;
  res =   s->p  | ((s->div - 2) << 2) | ( (s->mul - 2) << 8) ;
  DEBUG1("$$$$$$$$$$$ PLLCTRL0=%x",res) ;
  DEBUG1("$$$$$$$$$$$ FREQ=%d",(14318180*s->mul/(s->div*2^s->p))) ;

  return res; 
}


static void 
pll_set(struct kgi_display *dpy, kgi_uint cmd)
{
  GET_PRIVATE ; 
  VIDPLL = cmd ; 
} 
 

#include "pll.inc"                         



